@@ -2,22 +2,17 @@
 Create a NeXus file for serial crystallography datasets collected on I19-2 Tristan10M detector.
 """
 
-import sys
-import h5py
 import logging
-
-from pathlib import Path
+import sys
 from collections import namedtuple
+from pathlib import Path
 
-from .I19_2_params import goniometer_axes, tristan10M_params, source
-
-from .. import (
-    get_iso_timestamp,
-    get_nexus_filename,
-)
+import h5py
 
+from .. import get_iso_timestamp, get_nexus_filename
 from ..nxs_write.NexusWriter import call_writers
-from ..nxs_write.NXclassWriters import write_NXentry, write_NXnote, write_NXdatetime
+from ..nxs_write.NXclassWriters import write_NXdatetime, write_NXentry, write_NXnote
+from .I19_2_params import goniometer_axes, source, tristan10M_params
 
 # Define a logger object and a formatter
 logger = logging.getLogger("NeXusGenerator.I19-2_ssx")
@@ -116,14 +111,11 @@ def write_nxs(**ssx_params):
         0.0,
     ]
 
-    # Define SCANS dictionary
-    SCANS = {}
-
     # Get rotation scan range array and axis
     osc_axis = "phi"
     osc_range = (0.0, 0.0)
 
-    SCANS["rotation"] = {osc_axis: osc_range}
+    OSC = {osc_axis: osc_range}
 
     logger.info("Goniometer information")
     for j in range(len(goniometer["axes"])):
@@ -176,7 +168,6 @@ def write_nxs(**ssx_params):
                 nxsfile,
                 [metafile],
                 coordinate_frame,
-                SCANS,
                 (
                     detector["mode"],
                     None,
@@ -187,6 +178,8 @@ def write_nxs(**ssx_params):
                 source,
                 beam,
                 attenuator,
+                OSC,
+                transl_scan=None,
                 metafile=metafile,  # Since there are no links, this could also be None
                 link_list=None,
             )
