@@ -2,30 +2,28 @@
 Writer functions for different groups of a NeXus file.
 """
 
-import h5py
 import logging
-
-import numpy as np
-
-from pathlib import Path
 from datetime import datetime
-from typing import List, Dict, Tuple, Union, Optional
+from pathlib import Path
+from typing import Dict, List, Optional, Tuple, Union
+
+import h5py
+import numpy as np
+from hdf5plugin import Bitshuffle
 
-from . import (
-    calculate_origin,
-    create_attributes,
-    set_dependency,
-)
 from .. import (
+    get_iso_timestamp,
     imgcif2mcstas,
     split_arrays,
-    get_iso_timestamp,
     units_of_length,
     units_of_time,
     ureg,
 )
+from . import calculate_origin, create_attributes, set_dependency
 
 NXclass_logger = logging.getLogger("NeXusGenerator.writer.NXclass")
+NXclass_logger.setLevel(logging.DEBUG)
+
 
 # NXentry writer
 def write_NXentry(nxsfile: h5py.File, definition: str = "NXmx") -> h5py.Group:
@@ -76,7 +74,10 @@ def write_NXdata(
     """
     NXclass_logger.info("Start writing NXdata.")
     # Check that a valid datafile_list has been passed.
-    assert len(datafiles) > 0, "Please pass at least a list of one HDF5 data file."
+    if len(datafiles) == 0:
+        raise OSError(
+            "No HDF5 data filenames have been found. Please pass at least one."
+        )
 
     # This assumes that a rotation scan is always passed
     osc_axis, osc_range = list(osc_scan.items())[0]
@@ -99,8 +100,13 @@ def write_NXdata(
     # If mode is images, link to blank image data. Else go to events.
     if data_type[0] == "images":
         tmp_name = f"data_%0{6}d"
-        for n, filename in enumerate(datafiles):
-            nxdata[tmp_name % (n + 1)] = h5py.ExternalLink(filename.name, "data")
+        if datafiles[0].parent != Path(nxsfile.filename).parent:
+            # This is needed in case the new NeXus file is to be written in a different directory from the data, eg. processing/
+            for n, filename in enumerate(datafiles):
+                nxdata[tmp_name % (n + 1)] = h5py.ExternalLink(filename, "data")
+        else:
+            for n, filename in enumerate(datafiles):
+                nxdata[tmp_name % (n + 1)] = h5py.ExternalLink(filename.name, "data")
     elif data_type[0] == "events":
         if len(datafiles) == 1 and "meta" in datafiles[0].as_posix():
             meta = datafiles
@@ -116,8 +122,9 @@ def write_NXdata(
         else:
             nxdata["meta_file"] = h5py.ExternalLink(meta[0].name, "/")
     else:
-        raise SystemExit("Please pass a correct data_type (images or events)")
-        # sys.exit("Please pass a correct data_type (images or events)")
+        raise ValueError(
+            "Unknown data type. Please pass one value for data_type from : [images, events]"
+        )
 
     # Write rotation axis dataset
     ax = nxdata.create_dataset(osc_axis, data=osc_range)
@@ -409,29 +416,96 @@ def write_NXdetector(
 
     # If there is a meta file, a lot of information will be linked instead of copied
     if meta and detector["mode"] == "images":
+        NXclass_logger.info(f"Found metadata to {meta.name} datasets.")
         for ll in link_list[0]:
             nxdetector[ll] = h5py.ExternalLink(meta.name, detector[ll])
+    elif detector["mode"] == "events":
+        wd = Path(nxsfile.filename).parent
+        # Bad pixel mask
+        if detector["pixel_mask"]:
+            nxdetector.create_dataset(
+                "pixel_mask_applied", data=detector["pixel_mask_applied"]
+            )
+            NXclass_logger.info(
+                f"Looking for file {detector['pixel_mask']} in {wd.as_posix()}."
+            )
+            maskfile = [
+                wd / detector["pixel_mask"]
+                for f in wd.iterdir()
+                if detector["pixel_mask"] == f.name
+            ]
+            if maskfile:
+                NXclass_logger.info("Pixel mask file found in working directory.")
+                block_size = 0
+                with h5py.File(maskfile[0], "r") as mh:
+                    mask = mh["image"][()]
+                    nxdetector.create_dataset(
+                        "pixel_mask",
+                        data=mask,
+                        **Bitshuffle(nelems=block_size, lz4=True),
+                    )
+                NXclass_logger.info(
+                    "A compressed copy of the pixel mask has been written into the NeXus file."
+                )
+            else:
+                NXclass_logger.warning("No pixel mask file found in working directory.")
+                NXclass_logger.warning("Writing an ExternalLink.")
+                mask = Path(detector["pixel_mask"])
+                image_key = (
+                    "image" if "tristan" in detector["description"].lower() else "/"
+                )
+                nxdetector["pixel_mask"] = h5py.ExternalLink(mask.name, image_key)
+        # Flatfield
+        if detector["flatfield"]:
+            nxdetector.create_dataset(
+                "flatfield_applied", data=detector["flatfield_applied"]
+            )
+            NXclass_logger.info(
+                f"Looking for file {detector['flatfield']} in {wd.as_posix()}."
+            )
+            flatfieldfile = [
+                wd / detector["flatfield"]
+                for f in wd.iterdir()
+                if detector["flatfield"] == f.name
+            ]
+            if flatfieldfile:
+                NXclass_logger.info("Flatfield file found in working directory.")
+                block_size = 0
+                with h5py.File(flatfieldfile[0], "r") as mh:
+                    flatfield = mh["image"][()]
+                    nxdetector.create_dataset(
+                        "flatfield",
+                        data=flatfield,
+                        **Bitshuffle(nelems=block_size, lz4=True),
+                    )
+                NXclass_logger.info(
+                    "A compressed copy of the flatfield has been written into the NeXus file."
+                )
+            else:
+                NXclass_logger.warning(
+                    "No flatfield file found in th working directory."
+                )
+                NXclass_logger.warning("Writing an ExternalLink.")
+                flatfield = Path(detector["flatfield"])
+                image_key = (
+                    "image" if "tristan" in detector["description"].lower() else "/"
+                )
+                nxdetector["flatfield"] = h5py.ExternalLink(flatfield.name, image_key)
     else:
-        # Check for mask and flatfield files
-        # This is mostly for Tristan where there are external files
         # Flatfield
         if detector["flatfield"]:
             nxdetector.create_dataset(
                 "flatfield_applied", data=detector["flatfield_applied"]
             )
             flatfield = Path(detector["flatfield"])
-            image_key = "image" if "tristan" in detector["description"].lower() else "/"
-            nxdetector["flatfield"] = h5py.ExternalLink(flatfield.name, image_key)
-            # nxdetector.create_dataset("flatfield", data=detector["flatfield"])
+            nxdetector["flatfield"] = h5py.ExternalLink(flatfield.name, "/")
         # Bad pixel mask
         if detector["pixel_mask"]:
             nxdetector.create_dataset(
                 "pixel_mask_applied", data=detector["pixel_mask_applied"]
             )
             mask = Path(detector["pixel_mask"])
-            image_key = "image" if "tristan" in detector["description"].lower() else "/"
-            nxdetector["pixel_mask"] = h5py.ExternalLink(mask.name, image_key)
-            # nxdetector.create_dataset("pixel_mask", data=detector["pixel_mask"])
+            nxdetector["pixel_mask"] = h5py.ExternalLink(mask.name, "/")
 
     # Beam center
     # Check that the information hasn't already been written by the meta file.
@@ -598,7 +672,6 @@ def write_NXdetector_module(
         (
             "/entry/instrument/detector/transformations/detector_z/det_z",
             [0, 0, 0],
-            # offsets["fast_axis"],
             "mm",
             "translation",
             format(x_pix.units, "~"),
@@ -621,7 +694,6 @@ def write_NXdetector_module(
         (
             "/entry/instrument/detector/module/fast_pixel_direction",
             [0, 0, 0],
-            # offsets["slow_axis"],
             "mm",
             "translation",
             format(y_pix.units, "~"),
